@@ -124,6 +124,12 @@ get_beval_info(
 		*lnump = lnum;
 		*textp = lbuf;
 		*colp = col;
+#ifdef FEAT_VARTABS
+		vim_free(beval->vts);
+		beval->vts = tabstop_copy(wp->w_buffer->b_p_vts_array);
+		if (wp->w_buffer->b_p_vts_array != NULL && beval->vts == NULL)
+		    return FAIL;
+#endif
 		beval->ts = wp->w_buffer->b_p_ts;
 		return OK;
 	    }
@@ -137,7 +143,7 @@ get_beval_info(
  * Show a balloon with "mesg" or "list".
  */
     void
-post_balloon(BalloonEval *beval UNUSED, char_u *mesg, list_T *list)
+post_balloon(BalloonEval *beval UNUSED, char_u *mesg, list_T *list UNUSED)
 {
 # ifdef FEAT_BEVAL_TERM
 #  ifdef FEAT_GUI
@@ -269,10 +275,6 @@ general_beval_cb(BalloonEval *beval, int state UNUSED)
     if (bevalServers & BEVAL_NETBEANS)
 	netbeans_beval_cb(beval, state);
 #endif
-#ifdef FEAT_SUN_WORKSHOP
-    if (bevalServers & BEVAL_WORKSHOP)
-	workshop_beval_cb(beval, state);
-#endif
 
     recursive = FALSE;
 }
